import request from '@/helpers/request';
import { moneyFormat } from '@/helpers/utils';
import {
  Swipe,
  SwipeItem,
  Image,
  CellGroup,
  Cell,
  RadioGroup,
  Radio,
  Tag,
  Row,
  Col,
  ActionBarButton,
  Badge,
  SubmitBar,
  showImagePreview,
  showToast,
  Popup
} from 'vant';
import { defineComponent } from 'vue';
import styles from './index.module.less';
import AddGoodsCart from '../modal/add-goods-cart';
export default defineComponent({
  name: 'goods-detail',
  data() {
    const query = this.$route.query;
    return {
      id: query.id,
      albumPics: [] as any[],
      product: {} as Record<string | number | symbol, any>,
      radio: 0,
      skuStockListTemp: [],
      detailMobileHtml: '',
      loading: false,
      addGoodsShow: false,
      selectGoodsItem: {},
      cartCount: 0
    };
  },
  computed: {
    skuStockList() {
      // 处理规格
      const product = this.product;
      const skuStockList: any =
        this.skuStockListTemp.length > 0
          ? this.skuStockListTemp
          : [
              {
                id: -1,
                price: product.price,
                pic: product.pic,
                stock: product.stock,
                spData: null
              }
            ];
      skuStockList.forEach((item: any) => {
        if (item.spData) {
          const spData = JSON.parse(item.spData);
          item.spDataJson = spData.reduce(
            (spDataJson: any, value: { value: any }) => {
              spDataJson += value.value;
              return spDataJson;
            },
            ''
          );
          item.sku = spData
            .reduce((sku: string[], value: { key: any; value: any }) => {
              sku.push(`${value.key}: ${value.value}`);
              return sku;
            }, [])
            .join(',');
        } else {
          item.spDataJson = '默认';
        }
      });
      return skuStockList;
    },
    getPrice() {
      let item = this.skuStockList.filter(
        (n: { id: number }) => n.id == this.radio
      ) as any;
      if (item && Array.isArray(item) && item.length) {
        return item[0].price;
      }
      return 0;
    }
  },
  async mounted() {
    try {
      this.loading = true;
      const res = await request.get(
        `/edu-app/open/mall/product/detail/${this.id}`,
        {
          noAuthorization: true
        }
      );
      this.loading = false;
      const result = res.data || {};
      this.albumPics = [result.product.pic]
        .concat(result.product.albumPics.split(','))
        .filter(n => n);
      this.product = result.product;
      this.skuStockListTemp = result.skuStockList || [];
      if (this.skuStockListTemp.length) {
        let len = this.skuStockListTemp.length;
        for (let i = 0; i < len; i++) {
          let item = this.skuStockListTemp[i] as any;
          if (item.stock >= 0) {
            this.radio = item.id;
            break;
          }
        }
      }
      this.detailMobileHtml = result.product.detailMobileHtml;
    } catch {}
  },
  methods: {
    onPreview(index: number) {
      // 图片预览
      showImagePreview({
        images: this.albumPics,
        startPosition: index,
        closeable: true,
        className: styles.imagesOverlayWrap
      });
    },
    onShowImg(target: any) {
      const { localName } = target.srcElement;
      if (localName !== 'img') {
        return;
      }
      let startPosition = 0;
      const domList = document.querySelectorAll('.msgWrap img');
      let imgList = Array.from(domList).map((item: any, index: number) => {
        if (target.srcElement == item) {
          startPosition = index;
        }
        return item.src;
      });

      showImagePreview({
        images: imgList,
        startPosition: startPosition,
        closeable: true,
        className: styles.imagesOverlayWrap
      });
    },
    onShowCart() {
      this.selectGoodsItem = {
        price: this.product.price,
        originalPrice: this.product.originalPrice,
        pic: this.product.pic,
        stock: this.product.stock,
        skuStockList: this.skuStockListTemp.length
          ? this.skuStockListTemp
          : undefined,
        brandName: this.product.brandName,
        productCategoryId: this.product.productCategoryId,
        name: this.product.name,
        productSn: this.product.productSn,
        productSubTitle: this.product.subTitle,
        id: this.product.id
      };
      // 打开购物弹框
      this.addGoodsShow = true;
    }
  },
  render() {
    const product = this.product;
    const selectSku = this.skuStockList.find((n: any) => n.id === this.radio);
    return (
      <div class={styles.goodsDetail}>
        {/* <ColHeader /> */}
        <Swipe
          class={styles.swipe}
          lazyRender
          v-slots={{
            indicator: (item: any) =>
              item.total > 1 && (
                <div class={styles['custom-indicator']}>
                  {(item.active || 0) + 1} / {item.total}
                </div>
              )
          }}>
          {this.albumPics.map((item: string, index: number) => (
            <SwipeItem>
              <Image
                class={styles.swipeItemImg}
                src={item}
                onClick={() => this.onPreview(index)}
                fit="cover"
              />
            </SwipeItem>
          ))}
        </Swipe>

        <CellGroup border={false} class={[styles.goodsHead, 'mb12']}>
          <Cell
            center
            border={false}
            v-slots={{
              title: () => (
                <div class={styles.priceGroup}>
                  <span class={styles.price}>
                    <i>¥</i>
                    {moneyFormat(this.getPrice)}
                  </span>
                  {/* <del class={styles.delPrice}>
                    ¥{moneyFormat(product.originalPrice)}
                  </del> */}
                </div>
              )
              // default: () => <div class={styles.stock}>销量4件</div>
            }}
          />
          <Cell
            center
            border={false}
            title={product.name}
            titleClass={[styles.goodsName, 'van-ellipsis']}
          />
        </CellGroup>

        <Row class={[styles.row, 'mb12']}>
          <Col span={4} class={styles.col}>
            规格
          </Col>
          <Col span={20}>
            {selectSku ? (
              <div class={styles.selectWrap}>
                {selectSku.stock <= 0
                  ? `当前款式暂时缺货`
                  : `已选择 ${selectSku.spDataJson}`}
              </div>
            ) : (
              <div>请选择 规格</div>
            )}
            <RadioGroup
              class={styles['radio-group']}
              modelValue={this.radio}
              onUpdate:modelValue={val => (this.radio = val)}>
              {this.skuStockList.map((item: any) => {
                const isActive = item.id === this.radio;
                const type = isActive ? 'primary' : 'default';
                return (
                  <Badge
                    position="top-right"
                    content={item.stock <= 0 ? '缺货' : ''}
                    color={'#999999'}
                    class={styles.badge}
                    offset={[-20, 0]}>
                    <Radio
                      class={styles.radio}
                      name={item.id}
                      disabled={item.stock <= 0}
                      onClick={() => {
                        // 判断是否有库存
                        if (item.stock <= 0) {
                          return;
                        }
                        this.radio = item.id;
                      }}>
                      <Tag size="large" plain={isActive} type={type}>
                        {item.spDataJson}
                      </Tag>
                    </Radio>
                  </Badge>
                );
              })}
            </RadioGroup>
          </Col>
        </Row>
        {this.detailMobileHtml && (
          <div class={[styles.section]}>
            <div class={styles.detail}>
              <span>图文详情</span>
            </div>

            <div
              class={[styles.photoDetail, 'msgWrap']}
              onClick={this.onShowImg}
              v-html={this.detailMobileHtml}></div>
          </div>
        )}

        {!this.loading && (
          <>
            <SubmitBar
              class={styles.actionBar}
              safe-area-inset-bottom
              v-slots={{
                button: () => (
                  <div class={styles.buyGroup}>
                    <ActionBarButton
                      type="primary"
                      class={styles.addCertBtn}
                      text="添加购买"
                      onClick={() => this.onShowCart()}
                    />
                  </div>
                )
              }}></SubmitBar>
          </>
        )}

        <Popup
          show={this.addGoodsShow}
          closeable
          position="bottom"
          round
          onClose={() => {
            this.addGoodsShow = false;
          }}>
          <AddGoodsCart
            show={this.addGoodsShow}
            item={this.selectGoodsItem}
            onClose={() => {
              this.addGoodsShow = false;
            }}
            defaultRadio={this.radio}
            showType={'cart'}
          />
        </Popup>
      </div>
    );
  }
});
